<!DOCTYPE html>
<!--
Copyright 2015 Google Inc. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
-->

<html lang="en" ng-app="timesketch">

<head>
  <title>Timesketch</title>
  <meta name="csrf-token" content="{{ csrf_token() }}">
  <link rel="stylesheet" href="/static/third_party/font-awesome/css/font-awesome.min.css">
  <link rel="stylesheet" href="/static/third_party/bootstrap/css/bootstrap.min.css">
  <link rel="stylesheet" href="/static/css/ts.css">
  <link rel="shortcut icon" href="{{ url_for('static', filename='img/favicon.ico') }}">
  <script type="text/javascript" language="javascript" src="/static/third_party/jquery/jquery.min.js"></script>
  <script type="text/javascript" language="javascript" src="/static/third_party/bootstrap/js/bootstrap.min.js"></script>
  <script type="text/javascript" language="javascript" src="/static/third_party/moment/moment.min.js"></script>
  <script type="text/javascript" language="javascript" src="/static/third_party/angular/angular.js"></script>
  <script type="text/javascript" language="javascript" src="/static/app.js"></script>
  <script type="text/javascript" language="javascript" src="/static/components/api/api.js"></script>
  <script type="text/javascript" language="javascript" src="/static/components/api/api-service.js"></script>
  <script type="text/javascript" language="javascript" src="/static/components/explore/explore.js"></script>
  <script type="text/javascript" language="javascript" src="/static/components/explore/explore-event-directive.js"></script>
  <script type="text/javascript" language="javascript" src="/static/components/explore/explore-filter-directive.js"></script>
  <script type="text/javascript" language="javascript" src="/static/components/explore/explore-search-directive.js"></script>
  <script type="text/javascript" language="javascript" src="/static/components/core/core.js"></script>
  <script type="text/javascript" language="javascript" src="/static/components/core/core-butterbar-directive.js"></script>
  <script type="text/javascript" language="javascript" src="/static/components/core/core-upload-directive.js"></script>
</head>

<body>
<a href="/">
  <div id="logo">
    <img src="/static/img/logo.png" />
  </div>
</a>

<header>
  <div ts-butterbar id="butterbar">Loading..</div>

  <div class="pull-right" style="margin-top:5px;margin-right:30px;">
    {% block header_left %}{% endblock %}
    <span style="color:#fff;margin-right:20px;">{{ current_user.username }}</span>
    <a style="color:#fff;" href="/logout/">Logout</a>
  </div>
</header>

{% block sidenav %}{% endblock %}
{% block main_no_sidenav %}{% endblock %}

<div id="main">
  {% block main %}{% endblock %}
</div>

</body>
</html>